function  Res = maxsn12(data, beta, K, type)
% to compute the parameter estimator of models
% (1-beta)*SN(mu_1, lambda_1, sigma^2) + beta*SN(mu_2, lambda_2, sigma^2)
% beta is given mixing proportion
% data is N*1 vector
% K is the times of initialization
% type = 'homo' means 
%        theta = (mu_1, mu_2, lambda_1, lambda_2, sigma^2)
% type ='homo' means
%        theta = (mu_1, mu_2, lambda_1, lambda_2, sigma_1^2, sigma_2^2)

[N,kk] = size(data);
xm = mean(data);
xs = var(data);

switch type
    case 'homo'
    
    k1 = 0.4+0.6*rand(K,1);
    k2 = 0.4+0.6*rand(K,1);
    sig = xs./((1-beta)^2.*k1+beta^2.*k2);
    lam1 = sqrt(pi.*(1-k1)./(2-pi.*(1-k1)));
    lam2 = sqrt(pi.*(1-k2)./(2-pi.*(1-k2)));
    temp = xm-((1-beta).*sqrt(1-k1)+beta.*sqrt(1-k2)).*sig;
    mu1 = min(data)+(max(data)-min(data))*rand(K,1);
    mu2 = (temp-(1-beta).*mu1)./beta;
    paraint = [mu1 mu2 lam1 lam2 sig];

%paraint = [xm+randn(K,1) xm+randn(K,1) randn(K,1) randn(K,1) xs.*rand(K,1)];

% theta = (mu_1, mu_2, lambda_1, lambda_2, sigma^2)
    Fx = @(theta) N*log(sqrt(theta(5)))-...
        sum(log((1-beta).*normpdf((data-theta(1))./sqrt(theta(5)))...
        .*normcdf(theta(3).*(data-theta(1))./sqrt(theta(5)))...
        +beta.*normpdf((data-theta(2))./sqrt(theta(5)))...
        .*normcdf(theta(4).*(data-theta(2))./sqrt(theta(5)))))+...
        0.876*log(1+0.856*theta(3)^2)+0.876*log(1+0.856*theta(4)^2)...
        +0.25*(xs/theta(5)+log(theta(5)/xs));
    
    A = [0 0 0 0 -1;0 0 -1 0 0;0 0 1 0 0;0 0 0 -1 0;0 0 0 1 0];
    B = [-1e-6 100 100 100 100]';
    parahat = zeros(K, 6);
    for i=1:K
        %i
         [that,fval] = fmincon(Fx, paraint(i,:), A, B);
         if isreal(fval)
             parahat(i,:) = [that fval];
         else
             parahat(i,:) = Inf.*ones(1,6);
         end
    end

    [Pp, Ip] = min(parahat(:,6));
    paramin = parahat(Ip,:);
    Res = paramin(1,1:5);

    case 'heter'
    
    k1 = 0.4+0.6*rand(K,1);
    k2 = 0.4+0.6*rand(K,1);
    sig = xs./((1-beta)^2.*k1+beta^2.*k2);
    lam1 = sqrt(pi.*(1-k1)./(2-pi.*(1-k1)));
    lam2 = sqrt(pi.*(1-k2)./(2-pi.*(1-k2)));
    temp = xm-((1-beta).*sqrt(1-k1)+beta.*sqrt(1-k2)).*sig;
    mu1 = min(data)+(max(data)-min(data))*rand(K,1);
    mu2 = (temp-(1-beta).*mu1)./beta;
    paraint = [mu1 mu2 lam1 lam2 sig sig];

%paraint = [xm+randn(K,1) xm+randn(K,1) randn(K,1) randn(K,1) xs.*rand(K,1) xs.*rand(K,1)]

%theta = (mu_1, mu_2, lambda_1, lambda_2, sigma_1^2, sigma_2^2)
    Fx = @(theta) -sum(log((1-beta)/sqrt(theta(5)).*...
         normpdf((data-theta(1))./sqrt(theta(5))).*...
         normcdf(theta(3).*(data-theta(1))./sqrt(theta(5)))+...
         beta/sqrt(theta(6)).*normpdf((data-theta(2))./sqrt(theta(6))).*...
         normcdf(theta(4).*(data-theta(2))./sqrt(theta(6)))))+...
         0.876*log(1+0.856*theta(3)^2)+0.876*log(1+0.856*theta(4)^2)+...
         0.25*(xs/theta(5)+log(theta(5)/xs))+0.25*(xs/theta(6)+log(theta(6)/xs));

    
    A = [0 0 0 0 -1 0;0 0 0 0 0 -1;0 0 -1 0 0 0;0 0 1 0 0 0;0 0 0 -1 0 0;0 0 0 1 0 0];
    B = [-1e-6 -1e-6 100 100 100 100]';
    parahat = zeros(K, 7);
    for i=1:K
        %i
         [that, fval] = fmincon(Fx, paraint(i,:), A, B);
         if isreal(fval)
             parahat(i,:) = [that fval];
         else
             parahat(i,:) = Inf.*ones(1,7);
         end
    end

    [Pp, Ip] = min(parahat(:,7));
    paramin = parahat(Ip,:);
    Res = paramin(1,1:6);
    
    
    otherwise
        fprintf('Wrong in paramaters');
end