function [Logl, Mix, Nit, Lld] = plmesnm2(data, mix, options)
% compute penalized MLE of skew normal mixtures with ECM
% data is 1*N vector;
% mix is the structure variable contains 
%    mix.order: order of mixtures K
%    mix.pi: mixing proportions K*1
%    mix.mu: locations vector K*1
%    mix.sig: scale vector K*1
%    mix.lam: skewness vector K*1
% options(1) = iterating times bound
% options(2) = error tlerance rate
% Logl is log-likelihood;
% Nit is iterating times;
% Lld is a vector of all log-likelihood .
% only with p_n(\lambda) (Azzalini 2013)

% basic 
[Nk Nd] = size(data);
sn = var(data);
Lld = zeros(1,options(1));
err=1;
n=1;
logLold = psnmlog2(data, mix);

newmix.order = mix.order;
% Main loop
while (n<=options(1) && err>options(2))
%n
     % E step
     [Post, Ps1, Ps2] = snmpost(data, mix);
     
     % M step
     newmix.pi = sum(Post,2)./size(data,2);
     delam = mix.lam./sqrt(1+mix.lam.^2);
     newmix.mu = (Post*data'-sum(Ps1,2).*delam)./sum(Post,2);
     xmutemp = bsxfun(@minus, data, newmix.mu);
     %sum(sign(xmutemp),2)
     newmix.sig = (sum(Ps2,2)-2.*delam.*sum(Ps1.*xmutemp,2)+sum(Post.*xmutemp.^2,2))...
        ./(2.*(1-delam.^2).*sum(Post,2));

     % update mix.lam with ECM
     Ctemp = [newmix.sig.*sum(Post,2).*0.144 -0.144.*sum(xmutemp.*Ps1,2)...
         0.144.*(sum(Ps2,2)+sum(Post.*xmutemp.^2,2))-newmix.sig.*sum(Post,2).*1.144+1.5.*newmix.sig...
         0.856.*sum(xmutemp.*Ps1,2) newmix.sig.*(sum(Post,2)-1.5)-(sum(Ps2,2)+sum(Post.*xmutemp.^2,2))...
         sum(xmutemp.*Ps1,2)];
         
     mixtemp = newmix;
     mixtemp.lam = mix.lam;
     lamtemp = zeros(mix.order,1);
     logLtemp = psnmlog2(data, mixtemp);
     for tk = 1:mix.order
         
        delta =  roots(Ctemp(tk,:));
        delta1 = delta(imag(delta)==0);
        delta2 = delta1(-1<delta1 & delta1<1);
      
        if size(delta2,2)>0
            lam1 = delta2./sqrt(1-delta2.^2);
            Ltemp = zeros(size(delta2,2),1);
            for tj =1:size(delta2,2)
                mixtemp.lam(tk) = lam1(tj);
                Ltemp(tj) = psnmlog2(data, mixtemp);
            end
            [Vt Lt] = max(Ltemp);
            if Vt>=logLtemp
                lamtemp(tk) = lam1(Lt);
            else
                lamtemp(tk) = mix.lam(tk);
            end
        else
            lamtemp(tk) = mix.lam(tk);
        end
        mixtemp.lam = mix.lam;
     end
     newmix.lam = lamtemp;
     
%      myfun = @(x)newmix.sig.*(sum(Post,2)-1.5./(1-0.144.*x.^2)).*x.*(1-x.^2)+sum(xmutemp.*Ps1,2).*(1+x.^2)...
%            -(sum(Ps2,2)+sum(Post.*xmutemp.^2,2)).*x;
%      xlam = fsolve(myfun,mix.lam./sqrt(1+mix.lam.^2),optimset('Display','off','TolFun',1e-6));
%      xlam = min(xlam,temp);
%      xlam = max(xlam,-temp);
%      newmix.lam = xlam./sqrt(1-xlam.^2);
     
     % update variable
     logLnew = psnmlog2(data, newmix);
     err = abs((logLnew-logLold)/logLold);
     logLold = logLnew;
     Lld(n) = logLnew;
     mix = newmix;
     n = n + 1;
         
end

Logl = logLnew;
Mix = newmix;
Nit = n-1;
if Nit < options(1)
   Lld(n : options(1))=[];
end

     
