function [Post, Ps1, Ps2] = snmpost(data, mix)
% compute conditional expection in skew normal mixtures
% Post is z_{ij} of (12) in Lin (2007)
% Ps1 is s_{1ij} of (13) in Lin (2007)
% Ps2 is s_{2ij} of (14) in Lin (2007)
% Reference: Lin, T. I., Lee, J. C., & Yen, S. Y. (2007). 
% Finite mixture modelling using the skew normal distribution. 
% Statistica Sinica, 17(3), 909.

Temp0 = bsxfun(@minus, data, mix.mu);
Temp = bsxfun(@rdivide, Temp0, sqrt(mix.sig+realmin));
Temp1 = bsxfun(@times, Temp, mix.lam);

% approximate log(\Phi(x))
Lab1 = Temp1>-30;
Lab2 = Temp1<=-30;
Tempp = Temp1;
Tempp(Lab1) = log(normcdf(Temp1(Lab1)))-realmin;
Tempp(Lab2) = -Temp1(Lab2).^2./2 + 450 + log(normcdf(-30));

Temp2 = bsxfun(@plus, -Temp.^2./2+Tempp, log(mix.pi+realmin)-log(mix.sig+realmin)./2);
Tempe = exp(Temp2) + realmin;
Post = bsxfun(@rdivide, Tempe, sum(Tempe,1));

% approximate \phi(x)/\Phi(x) when x tends to -inf
Temppa = Temp1;
Temppa(Lab1) = normpdf(Temp1(Lab1))./normcdf(Temp1(Lab1))+realmin;
Temppa(Lab2) = -Temp1(Lab2);

Temp3 = bsxfun(@times, Temppa, sqrt((mix.sig+realmin)./(1+mix.lam.^2)));
mutemp = bsxfun(@times, Temp0, mix.lam./sqrt(1+mix.lam.^2));
Ps1 = Post.*(mutemp + Temp3);
Ps2 = Post.*(bsxfun(@plus, mutemp.^2, mix.sig./(1+mix.lam.^2)) + Temp3.*mutemp);


