function [Logl, Mix] = snmts(data, mix, post)
% compute MLE of skew normal mixtures with ECM (the two-step estimator)
% data is 1*N vector;
% mix is the structure variable contains 
%    mix.order: order of mixtures K
%    mix.pi: mixing proportions K*1
%    mix.mu: locations vector K*1
%    mix.sig: scale vector K*1
%    mix.lam: skewness vector K*1
% options(1) = iterating times bound
% options(2) = error tlerance rate
% Logl is log-likelihood;
% Nit is iterating times;
% Lld is a vector of all log-likelihood .

Mix = mix;

[postv, postl] = max(post,[],1);
centers = zeros(mix.order,1);
for i=1:mix.order
   dataT = data(postl==i);
   centers(i) = sum(dataT)/length(dataT);
end
   
Delta = (centers-mix.mu)./sqrt(2.*mix.sig./pi);
temp = (1-1e-3).*ones(mix.order,1);
Delta = min(Delta,temp);
Delta = max(Delta,-temp);
Lam = Delta./sqrt(1-Delta.^2);
Mix.lam = Lam;
Logl = snmlog(data, Mix);


     
